/*
 * @brineAveragePlot.java Version 1.1 05/19/2014
 *
 * Copyright (c) 2014 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.plot;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

import brine.brineListStruct;       // Brine Data List Structure

/** CLASS brineAveragePlot
 *  <p> This Class will create the Collins Bar Diagram.
 *
 *  @version 1.1 05/19/2014
 *  @author  John R. Victorine
 */

public class brineAveragePlot extends Canvas
{
  public static final int _K      = 0;  // Potassium
  public static final int _NA     = 1;  // Sodium
  public static final int _NA_K   = 2;  // Sodium/Potasium
  public static final int _MG     = 3;  // Magnesium
  public static final int _CA     = 4;  // Calcium
  public static final int _FE_II  = 5;  // Iron (II)

  public static final int _HCO3   = 6;  // Bicarbonate
  public static final int _CO3    = 7;  // Carbonate
  public static final int _SO4    = 8;  // Sulfate
  public static final int _CL     = 9;  // Chloride
  public static final int _BR     = 10; // Bromide
  public static final int _I      = 11; // Iodide

  public static final int _OHM    = 12; // Resistivity (Rw)
  public static final int _PH     = 13; // PH

  private brineListStruct stBrine  = null; // Common Brine Data
  private double          data[][] = null;

  private static final String sCATION[] = { "K", "Na", "Na-K", "Mg", "Ca", "Fe" };
  private static final String sANION[]  = { "HCO3", "CO3", "SO4", "Cl", "Br", "I" };
  private static final String sRw[]     = { "Rw" };
  private static final String sPH[]     = { "PH" };

  private int iWidth   = 1000; // Width of Plot
  private int iHeight  = 650;  // Height of Plot with the Titles

  private static final int _TOP  =  20; // Brine Concentration Plot Title
  private static final int _TOP2 =  40; // User Defined Title 1
  private static final int _TOP3 =  60; // User Defined Title 2

  /** CONSTRUCTOR brineAveragePlot()
   *  <p> This is the constructor for this class.
   *  @param stBrine = Brine Data List Structure
   */

  public brineAveragePlot( double data[][], brineListStruct stBrine )
  {
	this.data    = data;
	this.stBrine = stBrine;
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
	data    = null;
	stBrine = null;
  }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawTitle()
   *  <p> This method will draw the title Information
   */

  public void drawTitle( Graphics g )
  {
	int i=0;
	String sLabel    = "";
	String sName     = "";
	int    iName     = 0;
	String sAPI      = "";
	String sCounty   = "";
	int    iCounty   = 0;
	String state     = "";
	String sLocation = "";
	int    iLocation = 0;
	int    length    = 0;

    Font        fsb    = new Font("Monospaced", Font.BOLD, 16);
    FontMetrics fsbm   = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    sLabel = new String( "Brine Concentration Plot" );
    length = sLabel.length();
    g.drawString( sLabel, iWidth/2 - length*4 , _TOP );

    fsb  = new Font("Monospaced", Font.BOLD, 14);
    fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    sName     = "";
    iName     = 1;
	sCounty   = "";
	iCounty   = 1;
	sLocation = "";
	iLocation = 1;
	sLabel    = new String("");

	for (i=0; i<stBrine.iCount; i++)
	{
	  if (i==0)
	  {
		sName     = new String( stBrine.stItem[i].sName );
		sAPI      = new String( stBrine.stItem[i].sAPI );
	    sCounty   = new String( stBrine.stItem[i].sCounty );
	    state     = new String( stBrine.stItem[i].state );
	    sLocation = new String( stBrine.stItem[i].sLocation );
	  }

  	  if (!sName.equals(stBrine.stItem[i].sName))
	  {
	    iName = 0;
	  }

	  if (!sCounty.equals(stBrine.stItem[i].sCounty))
	  {
	    iCounty = 0;
	  }

	  if (!sLocation.equals(stBrine.stItem[i].sLocation))
	  {
	    iLocation = 0;
	  }
	}

	if ((iName == 1) && (sName.length() > 0))
	{
	  sLabel = new String( sName );

	  if (sAPI.length() > 0)
	  {
	    sLabel = new String( sLabel + " (" + sAPI + ")" );
	  }
	}
	else if ((iLocation == 1) && (sLocation.length() > 8))
	{
	  sLabel = new String( sLocation );

	  if (sCounty.length() > 0)
	  {
	    sLabel = new String( sLabel + ", " + sCounty );
	  }

	  if (state.length() > 0)
	  {
	    sLabel = new String( sLabel + ", " + state );
	  }
	}
	else if ((iCounty == 1) && (sCounty.length() > 0))
	{
	  sLabel = new String( sCounty );

      if (state.length() > 0)
      {
        sLabel = new String( sLabel + ", " + state );
      }
    }

    length = sLabel.length();
    g.drawString( sLabel, iWidth/2 - length*4 , _TOP2 );
  }

  /** Method drawFrame()
   *  <p> This method will draw a Frame around the plot area
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   */

  public void drawFrame( Graphics g,
                         int iXBegin, int iYBegin,
                         int iWide,   int iHigh )
  {
    g.setColor(Color.black);
    g.drawRect(iXBegin, iYBegin, iWide, iHigh);
  }

  /** Method drawXTitle()
   *  <p> This method will draw a X-Axis Title around the plot area
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param sText   - Array of X-Axis Label Text
   *  @param sYUnit  - Y-Axis Unit Text
   */

  public void drawXTitle( Graphics g,
                          int iXBegin, int iYBegin,
                          int iWide,   int iHigh,
                          String sText[], String sYUnit )
  {
	int iPos = iWide / sText.length;
	int len  = 0;

    g.setColor(Color.black);

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );

    len = sYUnit.length();
    g.drawString(sYUnit, iXBegin-8*len, iYBegin-14);

    g.setFont( new Font("Monospaced", Font.BOLD, 14) );

    for (int i=0; i<sText.length; i++)
    {
	  len = sText[i].length();

      g.drawString(sText[i], iXBegin+(i+1)*iPos-iPos/2-len*4, iYBegin-14);
	}
  }

  /** Method drawXLinearGrid()
   *  <p> This method will draw a X-Axis Linear Grid
   *  @param g           - Graphics pointer.
   *  @param iXBegin     - Starting point from left side of plot
   *  @param iYBegin     - Starting point from top of plot
   *  @param iWide       - Width of Plotting Area
   *  @param iHigh       - Height of Plotting Area
   *  @param dXLower     - X-Axis Start
   *  @param dXUpper     - X-Axis End
   *  @param dXIncrement - X-Axis Increment
   */

  public void drawXLinearGrid( Graphics g,
                               int iXBegin, int iYBegin,
                               int iWide,   int iHigh,
                               double dXLower, double dXUpper, double dXIncrement)
  {
	int i;
	int iTotal         = 0;
	double ratio       = 0.0;
	int    idraw       = 0;

    // Build the X Linear Grid

    iTotal = (int) Math.ceil((dXUpper-dXLower)/dXIncrement);

    for (i=0; i<iTotal; i++)
    {
      ratio = ((iWide * i * dXIncrement) / (dXUpper-dXLower));
      idraw = iXBegin + (int) ratio;

      if (idraw <= iXBegin+iWide+10)
      {
        g.setColor(Color.lightGray);
        g.drawLine(idraw, iYBegin, idraw, iYBegin+iHigh);
	  }
	}
  }

  /** Method drawYLinearGrid()
   *  <p> This method will draw a Y-Axis Linear Grid
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param dYLower     - Y-Axis Start
   *  @param dYUpper     - Y-Axis End
   *  @param dYIncrement - Y-Axis Increment
   */

  public void drawYLinearGrid( Graphics g,
                               int iXBegin, int iYBegin,
                               int iWide,   int iHigh,
                               double dYLower, double dYUpper, double dYIncrement)
  {
	int i;
	int iTotal         = 0;
	double ratio       = 0.0;
	int    idraw       = 0;
	double dy          = 0.0;
	String sText       ="";
	int    len         = 0;

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );

    // Build the Y Linear Grid

    iTotal = 1 + (int) Math.ceil((dYUpper-dYLower)/dYIncrement);

    for (i=0; i<iTotal; i++)
    {
      ratio = ((iHigh * i * dYIncrement) / (dYUpper-dYLower));
      idraw = iHigh + iYBegin - (int)ratio;
      g.setColor(Color.lightGray);
      g.drawLine(iXBegin, idraw, iXBegin+iWide, idraw);

      g.setColor(Color.black);
      dy = dYLower + (double) i * dYIncrement;
      dy = Math.round(dy*1000)/1000.0;
      sText = "" + (int)dy;
      len   = sText.length();
      g.drawString(sText, iXBegin-6*len, idraw+4);
	}
  }

  /** Method drawYLogGrid()
   *  <p> This method will draw a Y Axis Log Grid
   *  @param g        - Graphics pointer.
   *  @param iXBegin  - Starting point from left side of plot
   *  @param iYBegin  - Starting point from top of plot
   *  @param iWide    - Width of Plotting Area
   *  @param iHigh    - Height of Plotting Area
   *  @param dYLower  - Y-Axis Start
   *  @param dYCycles - Y-Axis Number of Log Cycles
   */

  public void drawYLogGrid( Graphics g,
                            int iXBegin, int iYBegin, //int iText,
                            int iWide,   int iHigh,
                            double dYLower, int iYCycle )
  {
	int i,j;
	int    iYinc   = 0;
	int    iMag    = 0;
	String sText   = "";
	int    len     = 0;
	double dTemp   = 0.0;
	int    jXinc   = 0;

    // Build the y Log Grid

    iYinc = iHigh / iYCycle;
    iMag  = (int) dYLower;

    for (i=iYBegin+iHigh; i>iYBegin+iYinc-10; i-=iYinc)
    {
      g.setColor(Color.lightGray);
      g.drawLine(iXBegin, i, iXBegin+iWide, i);

      // Draw the Minor Cycle Grid Lines Light Gray

      for (j=9; j>1; j--)
      {
        dTemp = 1.0 + iYinc * Math.log(j) / Math.log(10.0);
        jXinc = i - (int) dTemp;

        g.setColor(Color.lightGray);
        g.drawLine(iXBegin, jXinc, iXBegin+iWide, jXinc);
      }
    }

    g.setColor(Color.black);
    g.setFont( new Font("Monospaced", Font.BOLD, 10) );

    sText = "" + (int)dYLower;
    len   = sText.length();

    g.drawString("" + (int)dYLower, iXBegin-len*6, iYBegin+iHigh+4);

    for (i=iYBegin+iHigh; i>iYBegin; i-=iYinc)
    {
//        dy = (double) k * 10.0 * stPlot.dYMinimum;
//        k = k * 10;
      iMag  = iMag*10;
      sText = "" + iMag;
      len   = sText.length();

      g.drawString("" + iMag, iXBegin-len*6, i-iYinc+4);
    }
  }

  /** Method drawData()
   *  <p> This method will draw the data
   *  @param g        - Graphics pointer.
   *  @param iXBegin  - Starting point from left side of plot
   *  @param iYBegin  - Starting point from top of plot
   *  @param iWide    - Width of Plotting Area
   *  @param iHigh    - Height of Plotting Area
   *  @param iData    - Brine Data Average Data Identifier
   *  @param iTotal   - Total Number of Brine Data Identifiers
   *  @param iCount   - Count Variable - Start Value
   *  @param dYLower  - Y-Axis Start
   *  @param dYCycles - Y-Axis Number of Log Cycles
   */

  public void drawLogData( Graphics g,
                        int iXBegin, int iYBegin,
                        int iWide,   int iHigh,
                        int iData,   int iTotal,  int iCount,
                        double dYLower, int iYCycle)
  {
	double dMinimum = Math.log(dYLower) / Math.log(10);
	double dMaximum = dMinimum + iYCycle;

    double dMin     = 0.0;
	double d5       = 0.0;
	double d25      = 0.0;
	double d50      = 0.0;
	double dAvg     = 0.0;
    double d75      = 0.0;
    double d95      = 0.0;
    double dMax     = 0.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iPos     = iWide / iTotal;
	int    iStart   = 0;

	if (data != null)
	{
	  if (data[iData] != null)
	  {
		if (data[iData][0] > 0.0)
		{
	      dMin = Math.log(data[iData][brine.brineAvgStruct._MINIMUM]) / Math.log(10);
	      d5   = Math.log(data[iData][brine.brineAvgStruct._5]) / Math.log(10);
	      d25  = Math.log(data[iData][brine.brineAvgStruct._25]) / Math.log(10);
	      d50  = Math.log(data[iData][brine.brineAvgStruct._MEDIAN]) / Math.log(10);
	      dAvg = Math.log(data[iData][brine.brineAvgStruct._MEAN]) / Math.log(10);
	      d75  = Math.log(data[iData][brine.brineAvgStruct._75]) / Math.log(10);
	      d95  = Math.log(data[iData][brine.brineAvgStruct._95]) / Math.log(10);
	      dMax = Math.log(data[iData][brine.brineAvgStruct._MAXIMUM]) / Math.log(10);

          dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
          iMax   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
          i95    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
          i75    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
          i50    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
          iAvg   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
          i25    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
          i5     = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
          iMin   = iYBegin + iHigh - (int) dRatio;

          // Small Rectangle from 95% to 5%

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,102,0));
          g.fillRect(iStart, i95, 2, i5-i95);

          // Large Rectangle from 75% to 25%

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(136,201,155));
          g.fillRect(iStart, i75, iPos-6, i25-i75);

          // Medium

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,255,255));
          g.fillRect(iStart, i50-1, iPos-6, 2);

          // Mean

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart, iAvg-1, iPos-6, 2);

          // Minimum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMin, 8, 2);
          g.fillRect(iStart, iMin-3, 2, 8);

          // Maximum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMax, 8, 2);
          g.fillRect(iStart, iMax-3, 2, 8);
		}
	  }
	}
  }

  /** Method drawData()
   *  <p> This method will draw the data
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param iData   - Brine Data Average Data Identifier
   *  @param iTotal  - Total Number of Brine Data Identifiers
   *  @param iCount  - Count Variable - Start Value
   *  @param dYLower - Y-Axis Start
   *  @param dYUpper - Y-Axis End
   */

  public void drawData( Graphics g,
                        int iXBegin, int iYBegin,
                        int iWide,   int iHigh,
                        int iData,   int iTotal,  int iCount,
                        double dYLower, double dYUpper )
  {
	double dMinimum = dYLower;
	double dMaximum = dYUpper;

    double dMin     = 0.0;
	double d5       = 0.0;
	double d25      = 0.0;
	double d50      = 0.0;
	double dAvg     = 0.0;
    double d75      = 0.0;
    double d95      = 0.0;
    double dMax     = 0.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iPos     = iWide / iTotal;
	int    iStart   = 0;

	if (data != null)
	{
	  if (data[iData] != null)
	  {
		if (data[iData][0] > 0.0)
		{
	      dMin = data[iData][brine.brineAvgStruct._MINIMUM];
	      d5   = data[iData][brine.brineAvgStruct._5];
	      d25  = data[iData][brine.brineAvgStruct._25];
	      d50  = data[iData][brine.brineAvgStruct._MEDIAN];
	      dAvg = data[iData][brine.brineAvgStruct._MEAN];
	      d75  = data[iData][brine.brineAvgStruct._75];
	      d95  = data[iData][brine.brineAvgStruct._95];
	      dMax = data[iData][brine.brineAvgStruct._MAXIMUM];

          dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
          iMax   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
          i95    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
          i75    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
          i50    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
          iAvg   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
          i25    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
          i5     = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
          iMin   = iYBegin + iHigh - (int) dRatio;

          // Small Rectangle from 95% to 5%

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,102,0));
          g.fillRect(iStart, i95, 2, i5-i95);

          // Large Rectangle from 75% to 25%

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(136,201,155));
          g.fillRect(iStart, i75, iPos-6, i25-i75);

          // Medium

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,255,255));
          g.fillRect(iStart, i50-1, iPos-6, 2);

          // Mean

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart, iAvg-1, iPos-6, 2);

          // Minimum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMin, 8, 2);
          g.fillRect(iStart, iMin-3, 2, 8);

          // Maximum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMax, 8, 2);
          g.fillRect(iStart, iMax-3, 2, 8);
		}
	  }
	}
  }

  /** Method drawLegend()
   *  <p> This method will draw the Legend
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param dYLower - Y-Axis Start
   *  @param dYUpper - Y-Axis End
   */

  public void drawLegend( Graphics g,
                          int iXBegin, int iYBegin,
                          int iWide,   int iHigh,
                          double dYLower, double dYUpper )
  {
	double dMinimum = dYLower;
	double dMaximum = dYUpper;

    double dMin     = 1.0;
	double d5       = 2.0;
	double d25      = 3.0;
	double d50      = 4.0;
	double dAvg     = 5.0;
    double d75      = 6.0;
    double d95      = 7.0;
    double dMax     = 8.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iTotal   = 1;
	int    iPos     = iWide / iTotal;
	int    iStart   = 0;
	int    iCount   = 0;

    dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
    iMax   = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
    i95    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
    i75    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
    i50    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
    iAvg   = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
    i25    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
    i5     = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
    iMin   = iYBegin + iHigh - (int) dRatio;

    // Small Rectangle from 95% to 5%

    iStart = iXBegin + iCount * iPos + iPos/2 - 1;

    g.setColor(new Color(0,102,0));
    g.fillRect(iStart, i95, 2, i5-i95);

    g.setColor(new Color(0,0,0));
    g.drawString("Lower 5%", iXBegin+iPos, i5+4);

    g.setColor(new Color(0,0,0));
    g.drawString("Upper 95%", iXBegin+iPos, i95+4);

    // Large Rectangle from 75% to 25%

    iStart = iXBegin + iCount * iPos + 3;

    g.setColor(new Color(136,201,155));
    g.fillRect(iStart, i75, iPos-6, i25-i75);

    g.setColor(new Color(0,0,0));
    g.drawString("Lower 25%", iXBegin+iPos, i25+4);

    g.setColor(new Color(0,0,0));
    g.drawString("Upper 75%", iXBegin+iPos, i75+4);

    // Medium

    iStart = iXBegin + iCount * iPos + 3;

    g.setColor(new Color(0,255,255));
    g.fillRect(iStart, i50-1, iPos-6, 2);

    g.setColor(new Color(0,0,0));
    g.drawString("Median", iXBegin+iPos, i50+4);

    // Mean

    iStart = iXBegin + iCount * iPos + 3;

    g.setColor(new Color(0,0,0));
    g.fillRect(iStart, iAvg-1, iPos-6, 2);

    g.setColor(new Color(0,0,0));
    g.drawString("Mean", iXBegin+iPos, iAvg+4);

    // Minimum

    iStart = iXBegin + iCount * iPos + iPos/2 - 1;

    g.setColor(new Color(0,0,0));
    g.fillRect(iStart-3, iMin, 8, 2);
    g.fillRect(iStart, iMin-3, 2, 8);

    g.setColor(new Color(0,0,0));
    g.drawString("Minimum", iXBegin+iPos, iMin+4);

    // Maximum

    iStart = iXBegin + iCount * iPos + iPos/2 - 1;

    g.setColor(new Color(0,0,0));
    g.fillRect(iStart-3, iMax, 8, 2);
    g.fillRect(iStart, iMax-3, 2, 8);

    g.drawString("Maximum", iXBegin+iPos, iMax+4);
  }

  /** Method draw()
   * <p> This method will draw the stiff diagram
   *  @param g = Graphics pointer.
   */

  public void draw(Graphics g)
  {
	String sText       = "";
	int    len         = 0;
	int    iCount      = 0;

	int    iXCations   = 60;  // Starting Point from Left for the Cations
	int    iXAnions    = 360; // Starting Point from Left for the Anions
	int    iXRw        = 660; // Starting Point from Left for Rw
	int    iXPH        = 740; // Starting Point from Left for PH
	int    iXLegend    = 800; // Starting Point from Left for the Legend

	int    ionsXWide   = 240; // Width of Cations/Anions Plot Area
	int    imiscXWide  = 40;  // Width of Rw/PH Plot Area

	int    iYBegin     = 120; // Starting Point from top for all Plot Areas
	int    iYHigh      = 420; // Height of Plot Areas

    double dXStart     = 0.0; // X-Axis Start
    double dXEnd       = 0.0; // X-Axis End
    double dXIncrement = 0.0; // X-Axis Increment

    double dYStart     = 0.0; // Y-Axis Start
    double dYEnd       = 0.0; // Y-Axis End
    double dYIncrement = 0.0; // Y-Axis Increment
    int    iYCycles    = 0;   // Y-Axis Cycles if plot is log

    drawTitle( g );

    // Cations

    dXStart     = 0.0;
    dXEnd       = 6.0;
    dXIncrement = 1.0;

    dYStart     = 1.0;
    dYEnd       = 0;
    dYIncrement = 0;
    iYCycles    = 6;

    drawXLinearGrid( g, iXCations, iYBegin, ionsXWide, iYHigh, dXStart, dXEnd, dXIncrement );
    drawYLogGrid( g, iXCations, iYBegin, ionsXWide, iYHigh, dYStart, iYCycles );
    drawFrame( g, iXCations, iYBegin, ionsXWide, iYHigh );

    sText = "Cations";
	len   = sText.length();
    g.setFont( new Font("Monospaced", Font.BOLD, 14) );
    g.drawString(sText, iXCations + iYBegin - 4*len, 92);

    drawXTitle( g, iXCations, iYBegin, ionsXWide, iYHigh, sCATION, "mg/l" );

    iCount = 0;
    for (int i=_K; i<_HCO3; i++)
    {
	  drawLogData( g, iXCations, iYBegin, ionsXWide, iYHigh, i, 6, iCount, dYStart, iYCycles );
	  iCount++;
	}

    // Anions

    dXStart     = 0.0;
    dXEnd       = 6.0;
    dXIncrement = 1.0;

    dYStart     = 1.0;
    dYEnd       = 0;
    dYIncrement = 0;
    iYCycles    = 6;

    drawXLinearGrid( g, iXAnions, iYBegin, ionsXWide, iYHigh, dXStart, dXEnd, dXIncrement );
    drawYLogGrid( g, iXAnions, iYBegin, ionsXWide, iYHigh, dYStart, iYCycles );
    drawFrame( g, iXAnions, iYBegin, ionsXWide, iYHigh );

    sText = "Anions";
	len   = sText.length();
    g.setFont( new Font("Monospaced", Font.BOLD, 14) );
    g.drawString(sText, iXAnions + iYBegin - 4*len, 92);

    drawXTitle( g, iXAnions, iYBegin, ionsXWide, iYHigh, sANION, "mg/l" );

    iCount = 0;
    for (int i=_HCO3; i<_OHM; i++)
    {
	  drawLogData( g, iXAnions, iYBegin, ionsXWide, iYHigh, i, 6, iCount, dYStart, iYCycles );
	  iCount++;
	}

    // Rw

    dXStart     = 0.0;
    dXEnd       = 1.0;
    dXIncrement = 1.0;

    dYStart     = 1.0;
    dYEnd       = 0;
    dYIncrement = 0;
    iYCycles    = 3;

    drawXLinearGrid( g, iXRw, iYBegin, imiscXWide, iYHigh, dXStart, dXEnd, dXIncrement );
    drawYLogGrid( g, iXRw, iYBegin, imiscXWide, iYHigh, dYStart, iYCycles );
    drawFrame( g, iXRw, iYBegin, imiscXWide, iYHigh );

    drawXTitle( g, iXRw, iYBegin, imiscXWide, iYHigh, sRw, "Ohm-m" );

    drawLogData( g, iXRw, iYBegin, imiscXWide, iYHigh, _OHM, 1, 0, dYStart, iYCycles );

    // PH

    dXStart     = 0.0;
    dXEnd       = 1.0;
    dXIncrement = 1.0;

    dYStart     = 0.0;
    dYEnd       = 14.0;
    dYIncrement = 1.0;
    iYCycles    = 1;

    drawXLinearGrid( g, iXPH, iYBegin, imiscXWide, iYHigh, dXStart, dXEnd, dXIncrement );
    drawYLinearGrid( g, iXPH, iYBegin, imiscXWide, iYHigh, dYStart, dYEnd, dYIncrement );
    drawFrame( g, iXPH, iYBegin, imiscXWide, iYHigh );

    drawXTitle( g, iXPH, iYBegin, imiscXWide, iYHigh, sPH, "" );

    drawData( g, iXPH, iYBegin, imiscXWide, iYHigh, _PH, 1, 0, dYStart, dYEnd );

    // Legend

    dYStart     = 0.0;
    dYEnd       = 14.0;

    drawLegend( g, iXLegend, iYBegin, imiscXWide, iYHigh, dYStart, dYEnd );
  }

  /** Method paint( Graphics g )
   * <p> This method will paint the stiff diagram Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
    draw(g);
  }
}

/*
 *  @version 1.1 05/19/2014
 *  @author  John Victorine
 */
